from blueman.Functions import *
from blueman.Constants import *
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.main.Config import Config
from blueman.main.SignalTracker import SignalTracker
from blueman.bluez.Device import Device as BluezDevice
from blueman.bluez.Network import Network
from blueman.main.Device import Device
from blueman.Lib import rfcomm_list
from gi.repository import GObject
import weakref
import os
import cgi
from gi.repository import Gtk
from gi.repository import Pango
import dbus
import time
import datetime
import gettext


class MonitorBase(GObject.GObject):
    __gsignals__ = {
    'disconnected': (GObject.SignalFlags.NO_HOOKS, None, ()),
    'stats': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT,)),
    }

    def __init__(self, device, interface):
        GObject.GObject.__init__(self)

        self.interface = interface
        self.device = device
        self.config = Config("plugins/NetUsage/%s" % device.Address)

        self.last_tx = 0
        self.last_rx = 0

        if not self.config.props.tx:
            self.config.props.tx = "0"
        if not self.config.props.rx:
            self.config.props.rx = "0"
        if not self.config.props.time:
            self.config.props.time = int(time.time())

    #tx and rx must be cumulative absolute values
    def update_stats(self, tx, rx):
        dtx = tx - self.last_tx
        drx = rx - self.last_rx

        if dtx < 0:
            dtx = 0
        if drx < 0:
            drx = 0

        self.last_rx = rx
        self.last_tx = tx
        if dtx > 0:
            self.config.props.tx = str(int(self.config.props.tx) + dtx)
        if drx > 0:
            self.config.props.rx = str(int(self.config.props.rx) + drx)

        self.emit("stats", int(self.config.props.tx), int(self.config.props.rx))


    def Disconnect(self):
        self.emit("disconnected")


class NMMonitor(MonitorBase):
    def __init__(self, device, nm_dev_path):
        MonitorBase.__init__(self, device, "NM")
        dprint("created nm monitor for path", nm_dev_path)
        self.signals = SignalTracker()
        self.signals.Handle("dbus",
                            dbus.SystemBus(),
                            self.on_ppp_stats,
                            "PppStats",
                            "org.freedesktop.NetworkManager.Device.Serial",
                            path=nm_dev_path)

        self.signals.Handle(device, "property-changed", self.on_device_property_changed)

    def on_ppp_stats(self, rx, tx):
        self.update_stats(tx, rx)

    def on_device_property_changed(self, device, key, value):
        if key == "Connected" and not value:
            self.signals.DisconnectAll()
            self.Disconnect()


class Monitor(MonitorBase):
    def __init__(self, device, interface):
        MonitorBase.__init__(self, device, interface)
        self.poller = None

        self.poller = GObject.timeout_add(5000, self.poll_stats)

    def __del__(self):
        print("deleting monitor")

    def poll_stats(self):
        try:
            f = open("/sys/class/net/%s/statistics/tx_bytes" % self.interface, "r")
            tx = int(f.readline())
            f.close()

            f = open("/sys/class/net/%s/statistics/rx_bytes" % self.interface, "r")
            rx = int(f.readline())
            f.close()
        except IOError:
            self.poller = None
            self.ppp_port = None
            self.interface = None
            self.config = None
            self.Disconnect()
            return False

        self.update_stats(tx, rx)

        return True


class Dialog:
    running = False

    def __init__(self, parent):
        if not Dialog.running:
            Dialog.running = True
        else:
            return
        self.config = None
        self.parent = parent
        builder = Gtk.Builder()
        builder.add_from_file(UI_PATH + "/net-usage.ui")
        builder.set_translation_domain("blueman")

        self.dialog = builder.get_object("dialog")
        self.dialog.connect("response", self.on_response)
        cr1 = Gtk.CellRendererText()
        cr1.props.ellipsize = Pango.EllipsizeMode.END

        self.devices = {}
        self.signals = SignalTracker()

        self.signals.Handle(parent, "monitor-added", self.monitor_added)
        self.signals.Handle(parent, "monitor-removed", self.monitor_removed)
        self.signals.Handle(parent, "stats", self.on_stats)

        cr2 = Gtk.CellRendererText()
        cr2.props.sensitive = False
        cr2.props.style = Pango.Style.ITALIC

        self.liststore = Gtk.ListStore(str, str, str, object)

        self.e_ul = builder.get_object("e_ul")
        self.e_dl = builder.get_object("e_dl")
        self.e_total = builder.get_object("e_total")

        self.l_started = builder.get_object("l_started")
        self.l_duration = builder.get_object("l_duration")

        self.b_reset = builder.get_object("b_reset")
        self.b_reset.connect("clicked", self.on_reset)

        self.cb_device = builder.get_object("cb_device")
        self.cb_device.props.model = self.liststore
        self.cb_device.connect("changed", self.on_selection_changed)

        self.cb_device.pack_start(cr1, True)
        self.cb_device.add_attribute(cr1, 'markup', 1)

        self.cb_device.pack_start(cr2, False)
        self.cb_device.add_attribute(cr2, 'markup', 2)

        c = Config("plugins/NetUsage")
        devs = c.list_dirs()

        added = False
        for d in devs:
            d = os.path.basename(d)
            for m in parent.monitors:
                if d == m.device.Address:
                    iter = self.liststore.append(
                        [d, self.get_caption(m.device.Alias, m.device.Address), _("Connected:") + " " + m.interface, m])
                    if self.cb_device.get_active() == -1:
                        self.cb_device.set_active_iter(iter)
                    added = True
                    break
            if not added:
                name = d
                if self.parent.Applet.Manager:
                    for a in self.parent.Applet.Manager.ListAdapters():
                        try:
                            device = a.find_device(d)
                            device = Device(device)
                            name = self.get_caption(device.Alias, device.Address)
                        except:
                            pass

                self.liststore.append([d, name, _("Not Connected"), None])
            added = False
        if len(self.liststore) > 0:
            if self.cb_device.get_active() == -1:
                self.cb_device.set_active(0)
        else:
            d = Gtk.MessageDialog(parent=self.dialog, flags=Gtk.DialogFlags.MODAL, type=Gtk.MessageType.INFO,
                                  buttons=Gtk.ButtonsType.CLOSE, message_format=_(
                    "No usage statistics are available yet. Try establishing a connection first and then check this page."))
            d.props.icon_name = "blueman"
            d.run()
            d.destroy()
            self.on_response(None, None)
            return

        self.dialog.show()

    def on_response(self, dialog, response):
        self.signals.DisconnectAll()
        Dialog.running = False
        self.dialog.destroy()

    def update_time(self):
        time = self.config.props.time
        if time:
            self.datetime = datetime.datetime.fromtimestamp(time)

            self.l_started.props.label = str(self.datetime)

            delta = datetime.datetime.now() - self.datetime

            d = gettext.ngettext("day", "days", delta.days)
            h = gettext.ngettext("hour", "hours", delta.seconds / 3600)
            m = gettext.ngettext("minute", "minutes", delta.seconds % 3600 / 60)

            self.l_duration.props.label = _("%d %s %d %s and %d %s") % (
            delta.days, d, delta.seconds / 3600, h, delta.seconds % 3600 / 60, m)
        else:
            self.l_started.props.label = _("Unknown")
            self.l_duration.props.label = _("Unknown")

    def on_selection_changed(self, cb):
        iter = cb.get_active_iter()
        (addr,) = self.liststore.get(iter, 0)
        self.config = Config("plugins/NetUsage/" + addr)
        self.update_counts(self.config.props.tx, self.config.props.rx)
        self.update_time()

    def get_caption(self, name, address):
        return "%s\n<small>%s</small>" % (cgi.escape(name), address)

    def update_counts(self, tx, rx):
        tx = int(tx)
        rx = int(rx)

        (num, suffix) = format_bytes(tx)
        self.e_ul.props.text = "%.2f %s" % (num, suffix)

        (num, suffix) = format_bytes(rx)
        self.e_dl.props.text = "%.2f %s" % (num, suffix)

        (num, suffix) = format_bytes(int(tx) + int(rx))
        self.e_total.props.text = "%.2f %s" % (num, suffix)

        self.update_time()

    def on_reset(self, button):
        d = Gtk.MessageDialog(parent=self.dialog, flags=Gtk.DialogFlags.MODAL, type=Gtk.MessageType.QUESTION,
                              buttons=Gtk.ButtonsType.YES_NO,
                              message_format=_("Are you sure you want to reset the counter?"))
        res = d.run()
        d.destroy()
        if res == Gtk.ResponseType.YES:
            self.config.props.rx = "0"
            self.config.props.tx = "0"
            self.config.props.time = int(time.time())

            self.update_counts(0, 0)

    def on_stats(self, parent, monitor, tx, rx):
        iter = self.cb_device.get_active_iter()
        (mon,) = self.liststore.get(iter, 3)
        if mon == monitor:
            self.update_counts(tx, rx)

    def monitor_added(self, parent, monitor):
        for row in self.liststore:
            iter = row.iter
            (val,) = self.liststore.get(iter, 0)

            if val == monitor.device.Address:
                self.liststore.set(iter, 1, self.get_caption(monitor.device.Alias, monitor.device.Address), 2,
                                   _("Connected:") + " " + monitor.interface, 3, monitor)
                return

        self.liststore.append([monitor.device.Address, self.get_caption(monitor.device.Alias, monitor.device.Address),
                               _("Connected:") + " " + monitor.interface, monitor])

    def monitor_removed(self, parent, monitor):
        for row in self.liststore:
            iter = row.iter
            (val,) = self.liststore.get(iter, 0)

            if val == monitor.device.Address:
                self.liststore.set(iter, 1, self.get_caption(monitor.device.Alias, monitor.device.Address), 2,
                                   _("Not Connected"), 3, None)
                return


class NetUsage(AppletPlugin, GObject.GObject):
    __depends__ = ["Menu"]
    __icon__ = "network-wireless"
    __description__ = _(
        "Allows you to monitor your (mobile broadband) network traffic usage. Useful for limited data access plans. This plugin tracks every device seperately.")
    __author__ = "Walmis"
    __autoload__ = False
    __gsignals__ = {
    'monitor-added': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
    'monitor-removed': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
    #monitor, tx, rx
    'stats': (
    GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT,)),
    }

    def on_load(self, applet):
        GObject.GObject.__init__(self)
        self.monitors = []
        self.devices = weakref.WeakValueDictionary()
        self.signals = SignalTracker()

        bus = self.bus = dbus.SystemBus()
        self.signals.Handle('bluez', Network(), self.on_network_property_changed, 'PropertyChanged',
                            path_keyword="path")

        item = create_menuitem(_("Network Usage"), get_icon("network-wireless", 16))
        item.props.tooltip_text = _("Shows network traffic usage")
        self.signals.Handle(item, "activate", self.activate_ui)
        self.Applet.Plugins.Menu.Register(self, item, 84, True)

        self.signals.Handle("dbus", bus, self.on_nm_ppp_stats, "PppStats",
                            "org.freedesktop.NetworkManager.Device.Serial", path_keyword="path")
        self.nm_paths = {}

    def on_nm_ppp_stats(self, down, up, path):
        if not path in self.nm_paths:
            props = self.bus.call_blocking("org.freedesktop.NetworkManager",
                                           path,
                                           "org.freedesktop.DBus.Properties",
                                           "GetAll",
                                           "s",
                                           ["org.freedesktop.NetworkManager.Device"])

            if props["Driver"] == "bluetooth" and "rfcomm" in props["Interface"]:
                self.nm_paths[path] = True

                portid = int(props["Interface"].strip("rfcomm"))

                ls = rfcomm_list()
                for dev in ls:
                    if dev["id"] == portid:
                        adapter = self.Applet.Manager.get_adapter(dev["src"])
                        device = adapter.find_device(dev["dst"])
                        device = Device(device)

                        self.monitor_interface(NMMonitor, device, path)

                        return
            else:
                self.nm_paths[path] = False


    def on_network_property_changed(self, key, value, path):
        dprint(key, value, path)
        if key == "Interface" and value != "":
            d = BluezDevice(path)
            d = Device(d)
            self.monitor_interface(Monitor, d, value)

    def activate_ui(self, item):
        Dialog(self)

    def on_unload(self):
        self.signals.DisconnectAll()
        self.Applet.Plugins.Menu.Unregister(self)

    def monitor_interface(self, montype, *args):
        m = montype(*args)
        self.monitors.append(m)
        self.signals.Handle(m, "stats", self.on_stats, sigid=m)
        self.signals.Handle(m, "disconnected", self.on_monitor_disconnected, sigid=m)
        self.emit("monitor-added", m)

    def on_ppp_connected(self, device, rfcomm, ppp_port):
        self.monitor_interface(Monitor, device, ppp_port)

    def on_monitor_disconnected(self, monitor):
        self.monitors.remove(monitor)
        self.signals.Disconnect(monitor)
        self.emit("monitor-removed", monitor)

    def on_stats(self, monitor, tx, rx):
        self.emit("stats", monitor, tx, rx)



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
from gi.repository import Gtk
from blueman.plugins.AppletPlugin import AppletPlugin

class ExitItem(AppletPlugin):
	__depends__ = ["Menu"]
	__autoload__ = False
	__description__ = _("Adds an exit menu item to quit the applet")
	__author__ = "Walmis"
	__icon__ = "gtk-quit"
	
	def on_load(self, applet):
		item = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_QUIT, None)
		item.connect("activate", lambda x: Gtk.main_quit())
		applet.Plugins.Menu.Register(self, item, 100)
		
	def on_unload(self):
		self.Applet.Plugins.Menu.Unregister(self)



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from blueman.Constants import *

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf
import re
import os
import signal
import atexit
import sys
from subprocess import Popen, call
import subprocess
import commands
from gi.repository import GObject
import traceback
from blueman.Lib import sn_launcher
import __builtin__

GREEN = lambda (x): "\x1b[32;01m" + x + "\x1b[39;49;00m"
BLUE = lambda (x): "\x1b[34;01m" + x + "\x1b[39;49;00m"
BOLD = lambda (x): "\033[1m" + x + "\033[0m"
YELLOW = lambda (x): "\x1b[33;01m" + x + "\x1b[39;49;00m"

import fcntl, struct, termios

try:
    in_fg = os.getpgrp() == struct.unpack('h', fcntl.ioctl(0, termios.TIOCGPGRP, "  "))[0]
except:
    in_fg = False


def dprint(*args):
    #dont print if in the background
    if in_fg:

        s = ""
        for a in args:
            s += (str(a) + " ")
        co = sys._getframe(1).f_code

        fname = BOLD(co.co_name)

        print "_________"
        print "%s %s" % (fname, "(%s:%d)" % (co.co_filename, co.co_firstlineno))
        print s
        try:
            sys.stdout.flush()
        except IOError:
            pass


__builtin__.dprint = dprint

from blueman.main.AppletService import AppletService


def check_bluetooth_status(message, exitfunc, *args, **kwargs):
    try:
        applet = AppletService()
    except:
        print "Blueman applet needs to be running"
        exitfunc()
    if "PowerManager" in applet.QueryPlugins():
        if not applet.GetBluetoothStatus():

            d = Gtk.MessageDialog(None, type=Gtk.MessageType.ERROR)
            d.props.icon_name = "blueman"
            d.props.text = _("Bluetooth Turned Off")
            d.props.secondary_text = message

            d.add_button(Gtk.STOCK_QUIT, Gtk.ResponseType.NO)
            d.add_button(_("Enable Bluetooth"), Gtk.ResponseType.YES)
            resp = d.run()
            d.destroy()
            if resp != Gtk.ResponseType.YES:
                exitfunc()
            else:
                applet.SetBluetoothStatus(True, *args, **kwargs)


def wait_for_adapter(bluez_adapter, callback, timeout=1000):
    def on_prop_change(key, value):
        if key == "Powered" and value:
            GObject.source_remove(source)
            bluez_adapter.unhandle_signal(on_prop_change, "PropertyChanged")
            callback()

    def on_timeout():
        bluez_adapter.unhandle_signal(on_prop_change, "PropertyChanged")
        GObject.source_remove(source)
        dprint(YELLOW("Warning:"),
               "Bluez didn't provide 'Powered' property in a reasonable timeout\nAssuming adapter is ready")
        callback()

    props = bluez_adapter.get_properties()
    if props["Address"] != "00:00:00:00:00:00":
        callback()
        return

    source = GObject.timeout_add(timeout, on_timeout)
    bluez_adapter.handle_signal(on_prop_change, "PropertyChanged")


def startup_notification(name, desc=None, bin_name=None, icon=None):
    dpy = Gdk.Display.get_default()
    #FIXME this will work with GTK3
    #screen = dpy.get_default_screen().get_number()
    screen = Gdk.Screen.get_default().get_number()
    sn = sn_launcher(dpy, screen)
    sn.set_name(name)

    if bin_name:
        sn.set_binary_name(bin_name.encode('UTF-8'))
    if icon:
        sn.set_icon_name(icon.encode('UTF-8'))

    if desc:
        sn.set_description(desc.encode('UTF-8'))

    sn.initiate("", "", Gtk.get_current_event_time())

    return sn


def enable_rgba_colormap():
    #screen = Gdk.Display.get_default().get_default_screen()
    #colormap = screen.get_rgba_colormap()
    #if colormap == None:
    #	colormap = screen.get_rgb_colormap()
    #Gtk.widget_set_default_colormap(colormap)
    pass


def spawn(command, system=False, sn=None, reap=True, *args, **kwargs):
    def child_closed(pid, cond):
        dprint(command, "closed")
        if sn:
            sn.complete()

    if not system:
        if type(command) == list:
            command[0] = os.path.join(BIN_DIR, command[0])
        else:
            command = os.path.join(BIN_DIR, command)
    else:
        if type(command) == list:
            command[0] = os.path.expanduser(command[0])
        else:
            command = os.path.expanduser(command)

    env = os.environ

    if sn:
        id = sn.get_startup_id()
        env["DESKTOP_STARTUP_ID"] = id

    env["BLUEMAN_EVENT_TIME"] = str(Gtk.get_current_event_time())

    p = Popen(command, env=env, *args, **kwargs)
    if reap:
        GObject.child_watch_add(p.pid, child_closed)
    return p


def setup_icon_path():
    ic = Gtk.IconTheme.get_default()
    ic.prepend_search_path(ICON_PATH)


def get_icon(name, size=24, fallback="gtk-missing-image"):
    ic = Gtk.IconTheme.get_default()

    try:
        icon = ic.load_icon(name, size, 0)
    except:
        if not fallback:
            raise
        try:
            icon = ic.load_icon(fallback, size, 0)
        except:
            icon = ic.load_icon("gtk-missing-image", size, 0)

    if icon.props.width > size:
        new_w = size
        new_h = int(size * ( float(icon.props.width) / icon.props.height ))
        icon = icon.scale_simple(new_w, new_h, GdkPixbuf.InterpType.BILINEAR)

    if icon.props.height > size:
        new_w = int(size * ( float(icon.props.height) / icon.props.width ))
        new_h = size
        icon = icon.scale_simple(new_w, new_h, GdkPixbuf.InterpType.BILINEAR)

    return icon


def get_notification_icon(icon, main_icon="blueman"):
    main = get_icon(main_icon, 48)
    sub = get_icon(icon, 24)

    return composite_icon(main, [(sub, 24, 24, 255)])


def adapter_path_to_name(path):
    return re.search(".*(hci[0-9]*)", path).groups(0)[0]

#format error
def e_(msg):
    if isinstance(msg, Exception):
        return str(msg) + "\n" + traceback.format_exc()
    else:
        msg = str(msg)

        s = msg.split(": ")
        del s[0]
        return ": ".join(s)


def opacify_pixbuf(pixbuf, alpha):
    new = pixbuf.copy()
    new.fill(0x00000000)
    pixbuf.composite(new, 0, 0, pixbuf.props.width, pixbuf.props.height, 0, 0, 1, 1, GdkPixbuf.InterpType.BILINEAR, alpha)
    return new

#pixbuf, [(pixbuf, x, y, alpha), (pixbuf, x, y, alpha)]

def composite_icon(target, sources):
    target = target.copy()
    for source in sources:
        source[0].composite(target, source[1], source[2], source[0].get_width(), source[0].get_height(), source[1],
                            source[2], 1, 1, GdkPixbuf.InterpType.NEAREST, source[3])

    return target


def format_bytes(size):
    ret = 0.0
    size = float(size)
    suffix = ""
    if size < 1024:
        ret = size
        suffix = "B"
    elif size > 1024 and size < (1024 * 1024):
        ret = size / 1024
        suffix = "KB"
    elif size > (1024 * 1024) and size < (1024 * 1024 * 1024):
        ret = size / (1024 * 1024)
        suffix = "MB"
    else:
        ret = size / (1024 * 1024 * 1024)
        suffix = "GB"

    return (ret, suffix)


def create_menuitem(text, image):
    item = Gtk.ImageMenuItem.new_with_mnemonic(text)
    item.set_image(Gtk.Image.new_from_pixbuf(image))

    return item


def check_single_instance(id, unhide_func=None):
    print "%s version %s starting" % (id, VERSION)
    cachedir = os.path.expanduser("~/.cache/")
    if not os.path.exists(cachedir):
        os.mkdir(cachedir)
    lockfile = os.path.join(cachedir, "%s-%s" % (id, os.getuid()))

    def handler(signum, frame):
        if unhide_func:
            f = open(lockfile)
            f.readline()
            try:
                event_time = long(f.readline())
            except:
                event_time = 0
            f.close()
            unhide_func(event_time)


    signal.signal(signal.SIGUSR1, handler)

    if os.path.exists(lockfile):
        f = open(lockfile)
        try:
            pid = int(f.readline())
        except:
            pid = 0

        try:
            event_time = int(f.readline())
        except:
            event_time = 0
        f.close()
        if pid > 0:
            isrunning = os.path.exists("/proc/%s" % pid)
            if isrunning:
                try:
                    f = open("/proc/%s/cmdline" % pid)
                    cmdline = f.readline().replace("\0", " ")
                    f.close()
                except:
                    cmdline = None
            if not isrunning or (isrunning and id not in cmdline):
                print "Stale PID, overwriting"
                os.remove(lockfile)
            else:
                print "There is an instance already running"
                time = os.getenv("BLUEMAN_EVENT_TIME") or 0

                f = file(lockfile, "w")
                f.write("%s\n%s" % (str(pid), str(time)))
                f.close()

                os.kill(pid, signal.SIGUSR1)
                exit()
        else:
            os.remove(lockfile)

    try:
        fd = os.open(lockfile, os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0664)
        os.write(fd, "%s\n%s" % (str(os.getpid()), "0"))
        os.close(fd)
    except OSError:
        print "There is an instance already running"
        exit()

    atexit.register(lambda: os.remove(lockfile))


def have(t):
    out = call(["which", t], stdout=subprocess.PIPE)

    return out != 1


def mask_ip4_address(ip, subnet):
    masked_ip = ""
    for i in range(4):
        masked_ip += chr(ord(ip[i]) & ord(subnet[i]))

    return masked_ip



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

import os
import time
import dbus


class Mechanism(dbus.proxies.Interface):
	__inst__ = None
	
	def __new__(c):
		if not Mechanism.__inst__:
			Mechanism.__inst__ = object.__new__(c)
		
		return Mechanism.__inst__
	
	
	def __init__(self):
		self.bus = dbus.SystemBus()
		
		service = self.bus.get_object("org.blueman.Mechanism", "/", follow_name_owner_changes=True)
		dbus.proxies.Interface.__init__(self, service, "org.blueman.Mechanism")




# Copyright (c) 2001, 2002, 2003 Python Software Foundation
# Copyright (c) 2004-2008 Paramjit Oberoi <param.cs.wisc.edu>
# Copyright (c) 2007 Tim Lauridsen <tla@rasmil.dk>
# All Rights Reserved.  See LICENSE-PSF & LICENSE for details.

from ini import INIConfig
from config import BasicConfig, ConfigNamespace
from compat import RawConfigParser, ConfigParser, SafeConfigParser

__all__ = [
    'INIConfig', 'BasicConfig', 'ConfigNamespace',
    'RawConfigParser', 'ConfigParser', 'SafeConfigParser',
]



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from blueman.Constants import *
from gi.repository import Gtk, Gdk
import cairo
from gi.repository import GObject
import weakref

class LinearController(object):
	def get_value(self, input):
		return input
		
class BezierController(LinearController):
	def __init__(self, curvature=0.5, start=0.0, end=1.0):
		self.curvature = curvature
		self.start = start
		self.end = end
		
	def __b(self, t, p1, p2, p3):
		return (1-t)**2*p1 + 2*(1-t)*t*p2 + t**2*p3
		
	def get_value(self, input):
		return self.__b(input, self.start, self.curvature, self.end)

class AnimBase(GObject.GObject):
	__gsignals__ = {
		'animation-finished' : (GObject.SignalFlags.RUN_LAST, None, ()),
	}
	
	def __init__(self, state=1.0):
		GObject.GObject.__init__(self)
		self._source = None
		self._state = state
		self.frozen = False
		self.fps = 24.0
		self.controller = LinearController()
		
	def set_controller(self, cls, *params):
		self.controller = cls(*params)

	def _do_transition(self):
		if abs(self._end-self._start) < 0.000001:
			return False
		
		self._state += self._step_size	
		
		if self._end-self._start < 0:
			if self._state <= self._end:
				self._state = self._end
				self._state_changed(self._state)
				self._source = None
				self.emit("animation-finished")
				return False
		else:
			if self._state >= self._end:
				self._state = self._end
				self._state_changed(self._state)
				self._source = None
				self.emit("animation-finished")
				return False
		
		self._state_changed(self._state)
		return True

	def thaw(self):
		self.frozen = False
		self.on_frozen(self.frozen)
	
	def freeze(self):
		self.frozen = True
		self.on_frozen(self.frozen)
		
	def on_frozen(self, is_frozen):
		pass
			
	def animate(self, start=1.0, end=0.0, duration=1000):
		if self.frozen:
			self.emit("animation-finished")
			return
		
		self._state = start
		self._start = start
		self._end = end
		self._duration = duration		
		
		if self._source:
			GObject.source_remove(self._source)
		
		try:
			self._step_size = (end-start) / (self.fps * (duration/1000.0))
		except ZeroDivisionError:
			self._state = end

			return
			

		self._state_changed(self._state)
		self._source = GObject.timeout_add(int(1.0/self.fps*1000), self._do_transition)
			
		
	def _state_changed(self, state):
		self.state_changed(self.controller.get_value(state))
		
	def state_changed(self, state):
		pass
		
	def get_state(self):
		return self._state
		
	def set_state(self, state):
		self._state = state
		self._state_changed(state)
		
	def is_animating(self):
		return self._source != None
		


class TreeRowFade(AnimBase):
	def __init__(self, tw, path, columns=None):
		AnimBase.__init__(self, 1.0)
		self.tw = tw
		
		self.sig = self.tw.connect_after("draw", self.on_expose)
		
		self.row = Gtk.TreeRowReference.new(tw.props.model, path)
		self.stylecontext = tw.get_style_context()
		self.columns = None

	def unref(self):
		if self.sig != None:
			self.tw.disconnect(self.sig)
			self.sig = None			
			
	def get_iter(self):
		return self.tw.props.model.get_iter(self.row.get_path())
		
		
	def on_expose(self, widget, cr):
		if self.frozen:
			return

		if not self.row.valid():
			self.tw.disconnect(self.sig)
			self.sig = None
			return
			
		path = self.row.get_path()
		
		area = ()
		
		color = self.stylecontext.get_background_color(0)

		if not self.columns:
			columns = self.tw.get_columns()
		else:
			columns = self.columns
			
		for col in columns:
			rect = self.tw.get_background_area(path, col)
			Gdk.cairo_get_clip_rectangle(cr)
			cr.rectangle(rect.x, rect.y, rect.width, rect.height)
			cr.clip()

			cr.set_source_rgba((1.0/65535)*color.red, (1.0/65535)*color.green, (1.0/65535)*color.blue, 1.0-self.get_state())
			cr.set_operator(cairo.OPERATOR_OVER)
	 		cr.paint()
			
	def state_changed(self, state):
		self.tw.queue_draw()
		#print state
		
class TreeRowColorFade(TreeRowFade):
	def __init__(self, tw, path, color):
		TreeRowFade.__init__(self, tw, path, None)
		
		self.color = color
		
	def do_animation_finished(self):
		self.unref()
	
	def on_expose(self, widget, cr):
		if self.frozen:
			return

		if not self.row.valid():
			self.tw.disconnect(self.sig)
			self.sig = None
			return
			
		path = self.row.get_path()
		
		area = ()
		
		color = self.stylecontext.get_background_color(0)

		if not self.columns:
			columns = self.tw.get_columns()
		else:
			columns = self.columns
			
		for col in columns:
			rect = self.tw.get_background_area(path, col)
			cr.rectangle(rect.x, rect.y, rect.width, rect.height)
			cr.clip()

			cr.set_source_rgba((1.0/65535)*self.color.red, (1.0/65535)*self.color.green, (1.0/65535)*self.color.blue, 1.0-self.get_state())
			cr.set_operator(cairo.OPERATOR_OVER)
	 		cr.paint()

class CellFade(AnimBase):
	def __init__(self, tw, path, columns=None):
		AnimBase.__init__(self, 1.0)
		self.tw = tw
		
		self.frozen = False
		self.sig = tw.connect_after("draw", self.on_expose)
		self.row = Gtk.TreeRowReference.new(tw.props.model, path)
		self.selection = tw.get_selection()
		self.style = Gtk.rc_get_style(tw)
		self.stylecontext = tw.get_style_context()
		self.columns = []
		for i in columns:
			self.columns.append(self.tw.get_column(i))
			
	def unref(self):
		if self.sig != None:
			self.tw.disconnect(self.sig)
			self.sig = None		
			
	def get_iter(self):
		return self.tw.props.model.get_iter(self.row.get_path())
			
	def on_expose(self, widget, cr):
		if self.frozen:
			return

		if not self.row.valid():
			self.tw.disconnect(self.sig)
			self.sig = None
			return
			
		path = self.row.get_path()
		
		area = ()

		color = self.stylecontext.get_background_color(0)
			
		for col in self.columns:
			bg_rect = self.tw.get_background_area(path, col)
			rect = self.tw.get_cell_area(path, col)
			rect.y = bg_rect.y
			rect.height = bg_rect.height
			
			cr.rectangle(rect.x, rect.y, rect.width, rect.height)
			cr.clip()
			if not (rect.height == 0 or rect.height == 0):
				detail = "cell_even" if path[0] % 2 == 0 else "cell_odd"
				if self.tw.props.rules_hint:
					detail += "_ruled"
				
				selected = self.selection.get_selected()[1] and self.tw.props.model.get_path(self.selection.get_selected()[1]) == path

				Gtk.paint_flat_box(self.tw.get_style(),
						     cr,
						     Gtk.StateType.SELECTED if (selected) else Gtk.StateType.NORMAL,
						     0,
						     self.tw,
						     detail,
						     rect.x,
						     rect.y,
						     rect.width,
						     rect.height)

				#FIXME pixmap got lost during port to gtk3
				#cr.set_source_pixmap(pixmap, rect.x, rect.y)
				cr.paint_with_alpha(self.get_state())
			
	def state_changed(self, state):
		self.tw.queue_draw()
		#print state
		
class WidgetFade(AnimBase):
	def __init__(self, widget, color):
		AnimBase.__init__(self, 1.0)
		
		self.widget = widget
		self.color = color
		
		self.sig = widget.connect_after("draw", self.on_expose)

	def on_expose(self, window, cr):
		if not self.frozen:
			rect = self.widget.get_allocation()
			cr.rectangle(rect.x, rect.y, rect.width, rect.height)
			cr.clip()		
			cr.set_source_rgba((1.0/65535)*self.color.red, (1.0/65535)*self.color.green, (1.0/65535)*self.color.blue, 1.0-self.get_state())
			cr.set_operator(cairo.OPERATOR_OVER)
		 	cr.paint()		

	
	def state_changed(self, state):
		self.widget.queue_draw()		
	
					




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from gi.repository import Gtk
import dbus
from blueman.Constants import *
from blueman.plugins.ServicePlugin import ServicePlugin

from blueman.main.AppletService import AppletService
from blueman.main.BluezConfig import BluezConfig
from blueman.main.Config import Config
from blueman.Functions import dprint
from blueman.main.Mechanism import Mechanism


class Audio(ServicePlugin):
	__plugin_info__ = (_("Audio"), "audio-card")
	def on_load(self, container):
		
		self.Builder = Gtk.Builder()
		self.Builder.set_translation_domain("blueman")
		self.Builder.add_from_file(UI_PATH +"/services-audio.ui")
		self.widget = self.Builder.get_object("audio")
		
		self.ignored_keys = []
		
		container.pack_start(self.widget, True, True, 0)
		
		self.cb_a2dp = self.Builder.get_object("a2dp")
		self.cb_hsp = self.Builder.get_object("hsp")
		
		self.info = self.Builder.get_object("info")
		
		c = BluezConfig("audio.conf")
		try:
			opt = c.get("General", "Enable")
			opt = opt.split(",")
			if "Source" in opt:
				self.cb_a2dp.props.active = True
			if "Gateway" in opt:
				self.cb_hsp.props.active = True
		except:
			pass
		
		self.cb_a2dp.connect("toggled", self.on_cfg_changed)
		self.cb_hsp.connect("toggled", self.on_cfg_changed)
		
		return True
		
	def on_enter(self):
		self.widget.props.visible = True
		
	def on_leave(self):
		self.widget.props.visible = False

	
	def on_apply(self):
		if self.get_options() != []:
			vals = ["Sink"]
			if self.cb_a2dp.props.active:
				vals.append("Source")
			if self.cb_hsp.props.active:
				vals.append("Gateway")
			try:
				m = Mechanism()
				m.SetBluezConfig("audio.conf", "General", "Enable", ",".join(vals))
				m.SaveBluezConfig("audio.conf")
				m.RestartBluez()
			except dbus.DBusException, e:
				dprint(e)
			else:
				self.clear_options()
		
	def on_cfg_changed(self, cb):
		self.option_changed_notify(cb)

	
	def on_query_apply_state(self):
		opts = self.get_options()
		if opts == []:
			self.info.props.visible = False
			return False
		else:
			self.info.props.visible = True
			return True




from blueman.Constants import *
from blueman.Functions import dprint
from blueman.Sdp import *
from gi.repository import Gtk


class ManagerToolbar:
    def __init__(self, blueman):
        self.blueman = blueman

        self.blueman.List.connect("device-selected", self.on_device_selected)
        self.blueman.List.connect("device-property-changed", self.on_device_propery_changed)
        self.blueman.List.connect("adapter-changed", self.on_adapter_changed)
        self.blueman.List.connect("adapter-property-changed", self.on_adapter_property_changed)

        #toolbar = blueman.Builder.get_object("toolbar2")
        #for c in toolbar.get_children():
        #	c.set_expand(True)

        self.b_search = blueman.Builder.get_object("b_search")
        self.b_search.connect("clicked", lambda button: blueman.inquiry())

        self.b_bond = blueman.Builder.get_object("b_bond")
        self.b_bond.connect("clicked", self.on_action, self.blueman.bond)

        self.b_trust = blueman.Builder.get_object("b_trust")
        self.b_trust.connect("clicked", self.on_action, self.blueman.toggle_trust)
        self.b_trust.set_homogeneous(False)

        self.b_trust.props.label = _("Untrust")
        size = Gtk.Requisition()
        (size, nsize) = Gtk.Widget.get_preferred_size(self.b_trust)
        self.b_trust.props.label = _("Trust")
        size2 = Gtk.Requisition()
        (size2, nsize2) = Gtk.Widget.get_preferred_size(self.b_trust)

        self.b_trust.props.width_request = max(size.width, size2.width)

        self.b_remove = blueman.Builder.get_object("b_remove")
        self.b_remove.connect("clicked", self.on_action, self.blueman.remove)

        self.b_add = blueman.Builder.get_object("b_add")
        self.b_add.connect("clicked", self.on_action, self.blueman.add_device)

        self.b_setup = blueman.Builder.get_object("b_setup")
        self.b_setup.connect("clicked", self.on_action, self.blueman.setup)
        self.b_setup.set_homogeneous(False)

        self.b_send = blueman.Builder.get_object("b_send")
        self.b_send.props.sensitive = False
        self.b_send.connect("clicked", self.on_action, self.blueman.send)
        self.b_send.set_homogeneous(False)

        self.b_browse = blueman.Builder.get_object("b_browse")
        self.b_browse.props.sensitive = False
        self.b_browse.connect("clicked", self.on_action, self.blueman.browse)
        #self.b_browse.set_homogeneous(False)

        self.on_adapter_changed(blueman.List, blueman.List.GetAdapterPath())

    def on_action(self, button, func):
        device = self.blueman.List.GetSelectedDevice()
        if device != None:
            func(device)


    def on_adapter_property_changed(self, List, adapter, (key, value)):
        if key == "Discovering":
            if value:
                self.b_search.props.sensitive = False
            else:
                self.b_search.props.sensitive = True

    def on_adapter_changed(self, list, adapter_path):
        dprint("toolbar adapter", adapter_path)
        if adapter_path == None:
            self.b_search.props.sensitive = False
            self.update_send_browse(None)
        else:
            self.b_search.props.sensitive = True

    def on_device_selected(self, dev_list, device, iter):
        if device == None or iter == None:
            self.b_bond.props.sensitive = False
            self.b_remove.props.sensitive = False
            self.b_trust.props.sensitive = False
            self.b_setup.props.sensitive = False
            self.b_add.props.sensitive = False
        else:
            row = dev_list.get(iter, "bonded", "trusted", "fake")
            self.b_setup.props.sensitive = True
            if row["bonded"]:
                self.b_bond.props.sensitive = False
            else:
                self.b_bond.props.sensitive = True

            if row["trusted"]:
                self.b_trust.props.sensitive = True
                self.b_trust.props.icon_name = "blueman-untrust"
                self.b_trust.props.label = _("Untrust")

            else:
                self.b_trust.props.sensitive = True
                self.b_trust.props.icon_name = "blueman-trust"
                self.b_trust.props.label = _("Trust")

            if row["fake"]:
                self.b_remove.props.sensitive = False
                self.b_add.props.sensitive = True
                self.b_trust.props.sensitive = False
                self.b_bond.props.sensitive = True
            else:
                self.b_remove.props.sensitive = True
                self.b_add.props.sensitive = False

        self.update_send_browse(device)

    def update_send_browse(self, device):
        self.b_send.props.sensitive = False
        self.b_browse.props.sensitive = False
        if device != None and not device.Fake:
            for uuid in device.UUIDs:
                uuid16 = uuid128_to_uuid16(uuid)
                if uuid16 == OBEX_OBJPUSH_SVCLASS_ID:
                    self.b_send.props.sensitive = True

                if uuid16 == OBEX_FILETRANS_SVCLASS_ID:
                    self.b_browse.props.sensitive = True
        if device and device.Fake:
            self.b_send.props.sensitive = True


    def on_device_propery_changed(self, dev_list, device, iter, (key, value)):
        if dev_list.compare(iter, dev_list.selected()):
            if key == "Trusted" or key == "Paired":
                self.on_device_selected(dev_list, device, iter)

            elif key == "Fake":
                self.on_device_selected(dev_list, device, iter)
                self.update_send_browse(device)

            elif key == "UUIDs":
                self.update_send_browse(device)
				
		
		




from gi.repository import Gtk
from gi.repository import Pango
import cgi
from blueman.Functions import *
from blueman.gui.DeviceList import DeviceList


class DeviceSelectorList(DeviceList):
    def __init__(self, adapter=None):
        cr = Gtk.CellRendererText()
        cr.props.ellipsize = Pango.EllipsizeMode.END
        data = [
            #device picture
            ["found_pb", 'GdkPixbuf', Gtk.CellRendererPixbuf(), {"pixbuf": 0}, None,
             {"spacing": 0, "sizing": Gtk.TreeViewColumnSizing.AUTOSIZE}],
            ["device_pb", 'GdkPixbuf', Gtk.CellRendererPixbuf(), {"pixbuf": 1}, None],

            #device caption
            ["caption", str, cr, {"markup": 2}, None, {"expand": True}],

            ["bonded_icon", 'GdkPixbuf', Gtk.CellRendererPixbuf(), {"pixbuf": 3}, None],
            ["trusted_icon", 'GdkPixbuf', Gtk.CellRendererPixbuf(), {"pixbuf": 4}, None]

            #["connected", bool], #used for quick access instead of device.GetProperties
            #["bonded", bool], #used for quick access instead of device.GetProperties
            #["trusted", bool], #used for quick access instead of device.GetProperties
            #["fake", bool], #used for quick access instead of device.GetProperties,
            #fake determines whether device is "discovered" or a real bluez device
        ]

        DeviceList.__init__(self, adapter, data)
        self.props.headers_visible = False

    def row_setup_event(self, iter, device):
        if not device.Fake:
            self.row_update_event(iter, "Trusted", device.Trusted)
            self.row_update_event(iter, "Paired", device.Paired)
        self.row_update_event(iter, "Fake", device.Fake)
        self.row_update_event(iter, "Alias", device.Alias)
        self.row_update_event(iter, "Icon", device.Icon)

    def device_add_event(self, device):
        if device.Fake:
            self.PrependDevice(device)
        else:
            self.AppendDevice(device)

    def row_update_event(self, iter, key, value):
        if key == "Trusted":
            if value:
                self.set(iter, trusted_icon=get_icon("blueman-trust", 16))
            else:
                self.set(iter, trusted_icon=None)


        elif key == "Paired":
            if value:
                self.set(iter, bonded_icon=get_icon("gtk-dialog-authentication", 16))
            else:
                self.set(iter, bonded_icon=None)

        elif key == "Fake":
            if value:
                self.set(iter, found_pb=get_icon("gtk-search", 16))
            else:
                self.set(iter, found_pb=None)

        elif key == "Alias":
            self.set(iter, caption=cgi.escape(value))

        elif key == "Icon":
            self.set(iter, device_pb=get_icon(value, 16))
	
	
	
	
	
	



import dbus
import dbus.glib
import dbus.service
import os.path
from blueman.Functions import get_icon, dprint
from gi.repository import Gtk
from gi.repository import GObject
from gi.types import GObjectMeta
import cgi
import blueman.bluez as Bluez
from blueman.Sdp import *
from blueman.Constants import *
from blueman.gui.Notification import Notification

from blueman.bluez.Agent import Agent, AgentMethod


class AgentErrorRejected(dbus.DBusException):
    def __init__(self):
        dbus.DBusException.__init__(self, name="org.bluez.Error.Rejected")


class AgentErrorCanceled(dbus.DBusException):
    def __init__(self):
        dbus.DBusException.__init__(self, name="org.bluez.Error.Canceled")


class DummyGObjectMeta(dbus.service.InterfaceType, GObjectMeta):
    pass


class CommonAgent(GObject.GObject, Agent):
    __metaclass__ = DummyGObjectMeta
    __gsignals__ = {
    'released': (GObject.SignalFlags.NO_HOOKS, None, ()),
    }

    def __init__(self, status_icon, path):
        Agent.__init__(self, path)
        GObject.GObject.__init__(self)

        self.status_icon = status_icon
        self.dbus_path = path
        self.dialog = None
        self.n = None

    def build_passkey_dialog(self, device_alias, dialog_msg, is_numeric):
        def on_insert_text(editable, new_text, new_text_length, position):
            if not new_text.isdigit():
                editable.stop_emission("insert-text")

        builder = Gtk.Builder()
        builder.add_from_file(UI_PATH + "/applet-passkey.ui")
        builder.set_translation_domain("blueman")
        dialog = builder.get_object("dialog")

        dialog.props.icon_name = "blueman"
        dev_name = builder.get_object("device_name")
        dev_name.set_markup(device_alias)
        msg = builder.get_object("message")
        msg.set_text(dialog_msg)
        pin_entry = builder.get_object("pin_entry")
        show_input = builder.get_object("show_input_check")
        if (is_numeric):
            pin_entry.set_max_length(6)
            pin_entry.set_width_chars(6)
            pin_entry.connect("insert-text", on_insert_text)
            show_input.hide()
        else:
            pin_entry.set_max_length(16)
            pin_entry.set_width_chars(16)
            pin_entry.set_visibility(False)
        show_input.connect("toggled", lambda x: pin_entry.set_visibility(x.props.active))
        accept_button = builder.get_object("accept")
        pin_entry.connect("changed", lambda x: accept_button.set_sensitive(x.get_text() != ''))

        return (dialog, pin_entry)

    def get_device_alias(self, device_path):
        device = Bluez.Device(device_path)
        props = device.get_properties()
        address = props["Address"]
        name = props["Name"]
        alias = address
        if name:
            alias = "<b>%s</b> (%s)" % (cgi.escape(name), address)
        return alias


    def ask_passkey(self, device_path, dialog_msg, notify_msg, is_numeric, notification, ok, err):
        def on_notification_close(n, action):
            if action != "closed":
                self.dialog.present()
            else:
                if self.dialog:
                    self.dialog.response(Gtk.ResponseType.REJECT)
                #self.applet.status_icon.set_blinking(False)


        def passkey_dialog_cb(dialog, response_id):
            if response_id == Gtk.ResponseType.ACCEPT:
                ret = pin_entry.get_text()
                if is_numeric:
                    ret = int(ret)
                ok(ret)
            else:
                err(AgentErrorRejected())
            dialog.destroy()
            self.dialog = None

        alias = self.get_device_alias(device_path)
        notify_message = _("Pairing request for %s") % (alias)

        if self.dialog:
            dprint("Agent: Another dialog still active, cancelling")
            err(AgentErrorCanceled())

        self.dialog, pin_entry = self.build_passkey_dialog(alias, dialog_msg, is_numeric)
        if not self.dialog:
            dprint("Agent: Failed to build dialog")
            err(AgentErrorCanceled())

        if notification:
            Notification(_("Bluetooth Authentication"), notify_message, pixbuf=get_icon("blueman", 48),
                         status_icon=self.status_icon)
        #self.applet.status_icon.set_blinking(True)

        self.dialog.connect("response", passkey_dialog_cb)
        self.dialog.present()

    def __del__(self):
        dprint("Agent on path", self.dbus_path, "deleted")

    @AgentMethod
    def Release(self):
        dprint("Agent.Release")
        self.Cancel()
        self.remove_from_connection()
        self.emit("released")

    @AgentMethod
    def Cancel(self):
        dprint("Agent.Cancel")
        if self.dialog:
            self.dialog.response(Gtk.ResponseType.REJECT)
        try:
            self.n.close()
        except:
            pass


# noinspection PyPep8Naming
class AdapterAgent(CommonAgent):
    def __init__(self, status_icon, adapter, time_func):
        self.adapter = adapter
        self.n = None
        self.time_func = time_func

        adapter_name = os.path.basename(adapter.get_object_path())

        CommonAgent.__init__(self, status_icon, "/org/blueman/agent/adapter/" + adapter_name)

    @AgentMethod
    def RequestPinCode(self, device, ok, err):
        dprint("Agent.RequestPinCode")
        dialog_msg = _("Enter PIN code for authentication:")
        notify_msg = _("Enter PIN code")
        self.ask_passkey(device, dialog_msg, notify_msg, False, True, ok, err)
        if self.dialog:
            self.dialog.present_with_time(self.time_func())

    @AgentMethod
    def RequestPasskey(self, device, ok, err):
        dprint("Agent.RequestPasskey")
        dialog_msg = _("Enter passkey for authentication:")
        notify_msg = _("Enter passkey")
        self.ask_passkey(device, dialog_msg, notify_msg, True, True, ok, err)
        if self.dialog:
            self.dialog.present_with_time(self.time_func())

    @AgentMethod
    def DisplayPasskey(self, device, passkey, entered):
        dprint("Agent.DisplayPasskey")

    @AgentMethod
    def RequestConfirmation(self, device, passkey, ok, err):
        def on_confirm_action(n, action):
            #self.applet.status_icon.set_blinking(False)
            if action == "confirm":
                ok()
            else:
                err(AgentErrorRejected())

        dprint("Agent.RequestConfirmation")
        alias = self.get_device_alias(device)
        notify_message = _("Pairing request for:") + "\n%s" % alias
        if passkey:
            notify_message += "\n" + _("Confirm value for authentication:") + " <b>%s</b>" % passkey
        actions = [["confirm", _("Confirm"), "gtk-yes"], ["deny", _("Deny"), "gtk-no"]]

        Notification("Bluetooth", notify_message, 0,
                     actions, on_confirm_action,
                     pixbuf=get_icon("blueman", 48), status_icon=self.status_icon)

    @AgentMethod
    def RequestAuthorization(self, device, ok, err):
        self.RequestConfirmation(device, None, ok, err)

    @AgentMethod
    def Authorize(self, device, uuid, ok, err):

        def on_auth_action(n, action):
            dprint(action)

            #self.applet.status_icon.set_blinking(False)
            if action == "always":
                device = Bluez.Device(n._device)
                device.set("Trusted", True)
            if action == "always" or action == "accept":
                ok()
            else:
                err(AgentErrorRejected())

            self.n = None

        dprint("Agent.Authorize")
        alias = self.get_device_alias(device)
        uuid16 = uuid128_to_uuid16(uuid)
        service = uuid16_to_name(uuid16)
        notify_message = (_("Authorization request for:") + "\n%s\n" + _("Service:") + " <b>%s</b>") % (alias, service)
        actions = [["always", _("Always accept"), "blueman-trust"],
                   ["accept", _("Accept"), "gtk-yes"],
                   ["deny", _("Deny"), "gtk-no"]]

        n = Notification(_("Bluetooth Authentication"), notify_message, 0,
                         actions, on_auth_action,
                         pixbuf=get_icon("blueman", 48), status_icon=self.status_icon)
        n._device = device

    @AgentMethod
    def AuthorizeService(self, device, uuid, ok, err):
        self.Authorize(device, uuid, ok, err)


class GlobalAgent(AdapterAgent):
    def __init__(self, status_icon, time_func):
        self.n = None
        self.time_func = time_func
        CommonAgent.__init__(self, status_icon, '/org/blueman/agent/global')


# noinspection PyPep8Naming
class TempAgent(CommonAgent):
    def __init__(self, status_icon, path, time):
        CommonAgent.__init__(self, status_icon, path)
        self.time = time

    @AgentMethod
    def RequestPinCode(self, device, ok, err):
        dprint("Agent.RequestPinCode")
        dialog_msg = _("Enter PIN code for authentication:")
        notify_msg = _("Enter PIN code")
        self.ask_passkey(device, dialog_msg, notify_msg, False, False, ok, err)
        if self.dialog:
            self.dialog.present_with_time(self.time)

    @AgentMethod
    def RequestPasskey(self, device, ok, err):
        dprint("Agent.RequestPasskey")
        dialog_msg = _("Enter passkey for authentication:")
        notify_msg = _("Enter passkey")
        self.ask_passkey(device, dialog_msg, notify_msg, True, False, ok, err)
        if self.dialog:
            self.dialog.present_with_time(self.time)

    @AgentMethod
    def RequestConfirmation(self, device, passkey, ok, err):
        dprint("Agent.RequestConfirmation")
        alias = self.get_device_alias(device)

        dialog = Gtk.MessageDialog(buttons=Gtk.ButtonsType.YES_NO)
        dialog.props.use_markup = True
        dialog.props.icon_name = "gtk-dialog-authentication"
        dialog.props.secondary_use_markup = True
        dialog.props.title = _("Confirm value")
        dialog.props.text = _("Pairing with: %s") % alias
        dialog.props.secondary_text = _("Confirm value for authentication:") + "<b>%s</b>" % passkey
        resp = dialog.run()
        if resp == Gtk.ResponseType.YES:
            ok()
        else:
            err(AgentErrorRejected())

        dialog.destroy()



import dbus
from gi.repository import GObject
import traceback


class SignalTracker:
    def __init__(self):
        self._signals = []

    def Handle(self, *args, **kwargs):
        auto = not type(args[0]) == str
        if "sigid" in kwargs:
            sigid = kwargs["sigid"]
            del kwargs["sigid"]
        else:
            sigid = None

        if auto:
            obj = args[0]
            args = args[1:]
            from blueman.bluez.BlueZInterface import BlueZInterface
            if isinstance(obj, BlueZInterface):
                objtype = "bluez"
            elif isinstance(obj, GObject.GObject):
                objtype = "gobject"
            elif isinstance(obj, dbus.proxies.Interface):
                objtype = "dbus"
            else:
                raise Exception("Unknown object type")
        else:
            objtype = args[0]
            obj = args[1]
            args = args[2:]

        if objtype == "bluez":
            obj.handle_signal(*args, **kwargs)
        elif objtype == "gobject":
            args = obj.connect(*args)
        elif objtype == "dbus":
            if isinstance(obj, dbus.Bus):
                obj.add_signal_receiver(*args, **kwargs)
            else:
                print("Deprecated use of dbus signaltracker")
                traceback.print_stack()
                obj.bus.add_signal_receiver(*args, **kwargs)

        self._signals.append((sigid, objtype, obj, args, kwargs))

    def Disconnect(self, sigid):
        for sig in self._signals:
            (_sigid, objtype, obj, args, kwargs) = sig
            if sigid != None and _sigid == sigid:
                if objtype == "bluez":
                    obj.unhandle_signal(*args)
                elif objtype == "gobject":
                    obj.disconnect(args)
                elif objtype == "dbus":
                    if isinstance(obj, dbus.Bus):
                        if "path" in kwargs:
                            obj.remove_signal_receiver(*args, **kwargs)
                        else:
                            obj.remove_signal_receiver(*args)
                    else:
                        obj.bus.remove_signal_receiver(*args)

                self._signals.remove(sig)


    def DisconnectAll(self):
        for sig in self._signals:

            (sigid, objtype, obj, args, kwargs) = sig
            if objtype == "bluez":
                obj.unhandle_signal(*args)
            elif objtype == "gobject":
                obj.disconnect(args)
            elif objtype == "dbus":
                if isinstance(obj, dbus.Bus):
                    if "path" in kwargs:
                        obj.remove_signal_receiver(*args, **kwargs)
                    else:
                        obj.remove_signal_receiver(*args)
                else:
                    obj.bus.remove_signal_receiver(*args)

        self._signals = []




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
import dbus
import dbus.glib
import dbus.service
import inspect

class MethodAlreadyExists(Exception):
	pass
	
class DbusService(dbus.service.Object):
	def __init__(self, interface, path, bus=dbus.SessionBus):
		self.interface = interface
		self.path = path
		
		self.bus = bus()
		self.bus.request_name(self.interface)
		
		dbus.service.Object.__init__(self, self.bus, self.path)
		
	def add_method(self, func, dbus_interface=None, in_signature="", *args, **kwargs):
		if not dbus_interface:
			dbus_interface = self.interface
		
		name = func.__name__	

		if name in self.__class__.__dict__:
			raise MethodAlreadyExists
		
		cnt = 0
		a = inspect.getargspec(func)[0]
		a = ",".join(a[1:])

		#print name, a	
		exec \
"""def %(0)s(self, %(1)s):
	return self.%(0)s._orig_func(%(1)s)

		
%(0)s._orig_func = func
dec = dbus.service.method(dbus_interface, in_signature, *args, **kwargs)(%(0)s)""" % {"0": func.__name__, "1": a}

		
		setattr(self.__class__, name, dec)
		if not dbus_interface in self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__]:
			self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__][dbus_interface] = {}
		
		self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__][dbus_interface][name] = dec
	
	def add_signal(self, name, dbus_interface=None, signature="", *args, **kwargs):
		if not dbus_interface:
			dbus_interface = self.interface
		
		if name in self.__class__.__dict__:
			raise MethodAlreadyExists
		a = ""
		for i in range(len(dbus.Signature(signature))):
			a += ", arg%d" % i

		exec "def func(self%s): pass" % a
		func.__name__ = name

		dec = dbus.service.signal(dbus_interface, signature, *args, **kwargs)(func)
		setattr(self.__class__, func.__name__, dec)
		if not dbus_interface in self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__]:
			self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__][dbus_interface] = {}
		self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__][dbus_interface][func.__name__] = dec

		return getattr(self, func.__name__)
		
	def remove_registration(self, name):
		print "remove", name
		delattr(self.__class__, name)
		del self._dbus_class_table[self.__class__.__module__+"."+self.__class__.__name__][self.interface][name]	




from blueman.Functions import dprint
import inspect
import dbus.service
from blueman.bluez.BlueZInterface import BlueZInterface
import errors

__SIGNATURES__ = {
    'Release': ('', ''),
    'RequestPinCode': ('o', 's'),
    'RequestPasskey': ('o', 'u'),
    'DisplayPasskey': ('ouu', ''),
    'RequestConfirmation': ('ou', ''),
    'RequestAuthorization': ('o', ''),
    'Authorize': ('os', ''),
    'AuthorizeService': ('os', ''),
    'Cancel': ('', '')
}


def AgentMethod(func):
    global __SIGNATURES__
    try:
        signatures = __SIGNATURES__[func.__name__]
    except KeyError:
        raise errors.BluezUnavailableAgentMethodError('method name ' + func.__name__ + ' unavailable for agent')
    args = inspect.getargspec(func)[0]
    if len(args) - len(signatures[0]) == 3:
        async_callbacks = (args[-2], args[-1])
    else:
        async_callbacks = None

    if BlueZInterface.get_interface_version()[0] < 5:
        interface = 'org.bluez.Agent'
    else:
        interface = 'org.bluez.Agent1'

    warp = dbus.service.method(interface, in_signature=signatures[0], out_signature=signatures[1],
                               async_callbacks=async_callbacks)
    return warp(func)


class Agent(dbus.service.Object):
    def __init__(self, obj_path):
        self.__obj_path = obj_path
        dbus.service.Object.__init__(self, dbus.SystemBus(), obj_path)

    def get_object_path(self):
        return self.__obj_path

    @AgentMethod
    def Release(self):
        dprint('Release')

    @AgentMethod
    def RequestPinCode(self, device):
        dprint('RequestPinCode (%s)' % (device))

    @AgentMethod
    def RequestPasskey(self, device):
        dprint('RequestPasskey (%s)' % (device))

    @AgentMethod
    def DisplayPasskey(self, device, passkey, entered):
        dprint('DisplayPasskey (%s, %d)' % (device, passkey))

    @AgentMethod
    def RequestConfirmation(self, device, passkey):
        dprint('RequestConfirmation (%s, %d)' % (device, passkey))

    @AgentMethod
    def RequestAuthorization(self, device):
        dprint('RequestAuthorization (%s)' % device)

    @AgentMethod
    def Authorize(self, device, uuid):
        dprint('Authorize (%s, %s)' % (device, uuid))

    @AgentMethod
    def AuthorizeService(self, device, uuid):
        dprint('AuthorizeService (%s, %s)' % (device, uuid))

    @AgentMethod
    def Cancel(self):
        dprint('Cancel')



from PropertiesBlueZInterface import PropertiesBlueZInterface
from ServiceInterface import ServiceInterface
from errors import raise_dbus_error
import dbus
import xml.dom.minidom


class Device(PropertiesBlueZInterface):
    @raise_dbus_error
    def __init__(self, obj_path=None):
        if self.__class__.get_interface_version()[0] < 5:
            interface = 'org.bluez.Device'
        else:
            interface = 'org.bluez.Device1'
        super(Device, self).__init__(interface, obj_path)

    @raise_dbus_error
    def list_services(self):
        interfaces = []
        dbus_object = dbus.SystemBus().get_object('org.bluez', self.get_object_path())
        dbus_introspect = dbus.Interface(dbus_object, 'org.freedesktop.DBus.Introspectable')
        introspect_xml = dbus_introspect.Introspect()
        root_node = xml.dom.minidom.parseString(introspect_xml)
        for interface in root_node.getElementsByTagName('interface'):
            interface_name = interface.getAttribute('name')
            if interface_name != self.get_interface_name():
                methods = []
                for method in interface.getElementsByTagName('method'):
                    methods.append(method.getAttribute('name'))
                interfaces.append(ServiceInterface(interface_name, self.get_object_path(), methods))
        return interfaces

    @raise_dbus_error
    def pair(self, reply_handler=None, error_handler=None):
        # BlueZ 5 only!
        def ok():
            if callable(reply_handler):
                reply_handler()

        def err(err):
            if callable(error_handler):
                error_handler(err)

        self.get_interface().Pair(reply_handler=ok, error_handler=err)



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# NMMonitor: Monitors a selected device and emits a signal when it was disconnected via NetworkManager
from gi.repository import GObject
import dbus
from blueman.Functions import *

from blueman.main.SignalTracker import SignalTracker
from blueman.plugins.AppletPlugin import AppletPlugin


NM_DEVICE_STATE_UNKNOWN = 0,

#/* Initial state of all devices and the only state for devices not
# * managed by NetworkManager.
# *
# * Allowed next states:
# *   UNAVAILABLE:  the device is now managed by NetworkManager
# */
NM_DEVICE_STATE_UNMANAGED = 1

#/* Indicates the device is not yet ready for use, but is managed by
# * NetworkManager.  For Ethernet devices, the device may not have an
# * active carrier.  For WiFi devices, the device may not have it's radio
# * enabled.
# *
# * Allowed next states:
# *   UNMANAGED:  the device is no longer managed by NetworkManager
# *   DISCONNECTED:  the device is now ready for use
# */
NM_DEVICE_STATE_UNAVAILABLE = 2

#/* Indicates the device does not have an activate connection to anything.
# *
# * Allowed next states:
# *   UNMANAGED:  the device is no longer managed by NetworkManager
# *   UNAVAILABLE:  the device is no longer ready for use (rfkill, no carrier, etc)
# *   PREPARE:  the device has started activation
# */
NM_DEVICE_STATE_DISCONNECTED = 3

#/* Indicate states in device activation.
# *
# * Allowed next states:
# *   UNMANAGED:  the device is no longer managed by NetworkManager
# *   UNAVAILABLE:  the device is no longer ready for use (rfkill, no carrier, etc)
# *   FAILED:  an error ocurred during activation
# *   NEED_AUTH:  authentication/secrets are needed
# *   ACTIVATED:  (IP_CONFIG only) activation was successful
# *   DISCONNECTED:  the device's connection is no longer valid, or NetworkManager went to sleep
# */
NM_DEVICE_STATE_PREPARE = 4
NM_DEVICE_STATE_CONFIG = 5
NM_DEVICE_STATE_NEED_AUTH = 6
NM_DEVICE_STATE_IP_CONFIG = 7

#/* Indicates the device is part of an active network connection.
# *
# * Allowed next states:
# *   UNMANAGED:  the device is no longer managed by NetworkManager
# *   UNAVAILABLE:  the device is no longer ready for use (rfkill, no carrier, etc)
# *   FAILED:  a DHCP lease was not renewed, or another error
# *   DISCONNECTED:  the device's connection is no longer valid, or NetworkManager went to sleep
# */
NM_DEVICE_STATE_ACTIVATED = 8

#/* Indicates the device's activation failed.
# *
# * Allowed next states:
# *   UNMANAGED:  the device is no longer managed by NetworkManager
# *   UNAVAILABLE:  the device is no longer ready for use (rfkill, no carrier, etc)
# *   DISCONNECTED:  the device's connection is ready for activation, or NetworkManager went to sleep
# */
NM_DEVICE_STATE_FAILED = 9

if not HAL_ENABLED:
	raise ImportError("NMMonitor (deprecated) requires hal support")


class NMMonitor(AppletPlugin, GObject.GObject):
	__gsignals__ = {
		#args: udi
		'disconnected' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_STRING,)),
		#args: udi
		'modem-removed' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_STRING,)),
		#args: udi, bdaddr
		'modem-added' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_STRING, GObject.TYPE_STRING,)),
	}
	
	__icon__ = "network"
	__description__ = _("Monitors NetworkManager's modem connections and automatically disconnects Bluetooth link after the network connection is closed")
	__author__ = "Walmis"
	
	def on_load(self, applet):
		GObject.GObject.__init__(self)

		self.bus = dbus.SystemBus()
		obj = self.bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
		
		self.hal_mgr = dbus.Interface(obj, 'org.freedesktop.Hal.Manager')
		
		self.monitored_udis = []
		
		self.signals = SignalTracker()
		
		
		self.signals.Handle("dbus", self.bus, self.on_device_state_changed, "StateChanged", "org.freedesktop.NetworkManager.Device", path_keyword="udi")
		self.signals.Handle("dbus", self.bus, self.on_device_added, "DeviceAdded", "org.freedesktop.Hal.Manager")
		self.signals.Handle("dbus", self.bus, self.on_device_removed, "DeviceRemoved", "org.freedesktop.Hal.Manager")
#		self.signals.Handle("bluez", device.Device, self.on_device_propery_changed, "PropertyChanged")
		
	def on_unload(self):
		self.signals.DisconnectAll()	
	
	def on_device_removed(self, udi):
		if udi in self.monitored_udis:
			self.monitored_udis.remove(udi)
			self.emit("modem-removed", udi)
		
	def on_device_added(self, udi):
		obj = self.bus.get_object('org.freedesktop.Hal', udi)
		device = dbus.Interface(obj, 'org.freedesktop.Hal.Device')		
		try:
			if device.QueryCapability("modem") and device.GetPropertyString("info.linux.driver") == "rfcomm":
				self.monitored_udis.append(udi)
				self.emit("modem-added", udi, device.GetPropertyString("info.bluetooth_address"))
		except:
			pass
						
	def on_device_state_changed(self, state, prev_state, reason, udi):
		if udi in self.monitored_udis:
			dprint("state=%u prev_state=%u reason=%u" % (state, prev_state, reason))
			if state <= 3 and 3 < prev_state <= 8:
				self.emit("disconnected", udi)
						



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
from blueman.Functions import *
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.bluez.Device import Device
from blueman.gui.Notification import Notification
from blueman.main.Mechanism import Mechanism
from blueman.main.PPPConnection import PPPConnection
from blueman.main.Config import Config

from gi.repository import GObject

from blueman.Sdp import *
import os
import dbus

class connection:
		
	def __init__(self, applet, device, port, ok, err):
		self.reply_handler = ok
		self.error_handler = err
		self.device = device
		self.port = port
		self.Applet = applet
		
		res = os.popen("ps x -o pid,args | grep modem-manager").read()
		if not res:
			self.connect()
		else:
			dprint("ModemManager is running, delaying connection 5sec for it to complete probing")
			GObject.timeout_add(5000, self.connect)
		
	def connect(self):
		c = Config("gsm_settings/" + self.device.Address)
		if c.props.apn == None:
			c.props.apn = ""
			
		if c.props.number == None:
			c.props.number = "*99#"	
		
		m = Mechanism()
		m.PPPConnect(self.port, c.props.number, c.props.apn, reply_handler=self.on_connected, error_handler=self.on_error, timeout=200)	
	
	def on_error(self, error):
		self.error_handler(error)
		GObject.timeout_add(1000, self.device.Services["serial"].Disconnect, self.port)
		
	def on_connected(self, iface):
		self.reply_handler(self.port)
		self.Applet.Plugins.Run("on_ppp_connected", self.device, self.port, iface)
		
		Notification(_("Connected"), _("Successfully connected to <b>DUN</b> service on <b>%(0)s.</b>\nNetwork is now available through <b>%(1)s</b>") % {"0":self.device.Alias, "1":iface}, pixbuf=get_icon("network-wireless", 48), status_icon=self.Applet.Plugins.StatusIcon)	

	

class PPPSupport(AppletPlugin):
	__depends__ = ["DBusService"]
	__conflicts__ = ["NMIntegration"]
	__description__ = _("Provides basic support for connecting to the internet via DUN profile.")
	__author__ = "Walmis"
	__icon__ = "modem"
	__priority__ = 0
	
	def on_load(self, applet):
		AppletPlugin.add_method(self.on_ppp_connected)
		
	def on_unload(self):
		pass
		
	def on_ppp_connected(self, device, rfcomm, ppp_port):
		pass
		
	def on_rfcomm_connected(self, device, port, uuid):
		pass
		
	def rfcomm_connect_handler(self, device, uuid, reply, err):
		uuid16 = sdp_get_serial_type(device.Address, uuid)
		if DIALUP_NET_SVCLASS_ID in uuid16:
		
			def local_reply(port):
				connection(self.Applet, device, port, reply, err)
				#self.port2device[port] = device
					
			device.Services["serial"].Connect(uuid, reply_handler=local_reply, error_handler=err)
			dprint("Connecting rfcomm device")
		
			return True
		else:
			return False
